import React from "react";
import Button from "@mui/material/Button";
import DialogActions from "@mui/material/DialogActions";
import DialogContent from "@mui/material/DialogContent";

import Accordion from "@mui/material/Accordion";
import AccordionSummary from "@mui/material/AccordionSummary";
import Box from "@mui/material/Box";
import Typography from "@mui/material/Typography";
import Stack from "@mui/material/Stack";

function PipelineTrackSelector(props) {
  const { cache, setCache, back, next } = props;

  const pipelineTracks = [];
  for (var track of cache.primarySelectedMappings) {
    for (var i in track) {
      if (!pipelineTracks[i]) pipelineTracks[i] = [];
      if (pipelineTracks[i].includes(track[i])) continue;
      pipelineTracks[i].push(track[i]);
    }
  }

  const addTrack = (test) => {
    const pipelineMasterTrack = cache.pipelineMasterTrack ?? [];
    for (var track of cache.primarySelectedMappings) {
      const trackIndex = track.indexOf(test);
      if (trackIndex === -1) continue;
      const trackSlice = track.slice(0, trackIndex + 1);
      trackSlice.forEach((t, i) => {
        if (!pipelineMasterTrack[i]) pipelineMasterTrack[i] = [];
        if (!pipelineMasterTrack[i].includes(t)) pipelineMasterTrack[i].push(t);
      });
      break;
    }

    console.log(pipelineMasterTrack);
    setCache({ ...cache, pipelineMasterTrack });
  };

  const removeTrack = (test) => {
    const pipelineMasterTrack = cache.pipelineMasterTrack ?? [];
    const toRemove = [];
    for (var mapping of cache.primarySelectedMappings) {
      const testIndex = mapping.indexOf(test);
      if (testIndex === -1) continue;
      const removeSlice = mapping.slice(testIndex, mapping.length);
      toRemove.push(...removeSlice);
    }

    for (var i in pipelineMasterTrack) {
      pipelineMasterTrack[i] = pipelineMasterTrack[i].filter(
        (t) => !toRemove.includes(t)
      );
    }
    setCache({ ...cache, pipelineMasterTrack });
  };

  const selectTrack = (test) => () => {
    const pipelineMasterTrack = cache.pipelineMasterTrack ?? [];
    if (![].concat.apply([], pipelineMasterTrack).includes(test))
      return addTrack(test);
    removeTrack(test);
  };

  const getColor = (test) => {
    return [].concat.apply([], cache.pipelineMasterTrack).includes(test)
      ? "primary"
      : null;
  };

  return (
    <React.Fragment>
      <DialogContent>
        <h3>Select Track</h3>
        {pipelineTracks.map((track, i) => (
          <React.Fragment key={i}>
            <Typography variant="h6">{i + 1}</Typography>
            <Box>
              {track.map((test, j) => (
                <Accordion
                  expanded={false}
                  disableGutters={true}
                  square
                  key={j}
                  onClick={selectTrack(test)}
                >
                  <AccordionSummary
                    style={{
                      backgroundColor: "rgba(0, 0, 0, .03)",
                      flexWrap: "wrap",
                    }}
                  >
                    <Typography
                      component={"span"}
                      style={{ wordBreak: "break-word", margin: "auto 0" }}
                      color={getColor(test)}
                    >
                      {test}
                    </Typography>
                    <Stack sx={{ ml: "auto" }}>I</Stack>
                  </AccordionSummary>
                </Accordion>
              ))}
            </Box>
          </React.Fragment>
        ))}
      </DialogContent>
      <DialogActions>
        <Button onClick={back}>Back</Button>
        <Button onClick={next} autoFocus>
          Next
        </Button>
      </DialogActions>
    </React.Fragment>
  );
}

export default PipelineTrackSelector;
